      PROGRAM GYRORT
  


C            VERSION 1             14 FEBRUARY 2001
C
C    Address questions and comments to
C         Gordon D. Holman
C         NASA/Goddard Space Flight Center
C         Code 682
C         Greenbelt, MD 20771
C         Office phone: (301) 286-4636
C		E-Mail: holman@stars.gsfc.nasa.gov
C
C    Purpose -
C         This program computes gyrosynchrotron spectra by integrating
C		the equation of radiative transfer along a line of sight 
C		through an inhomogeneous radio source.  Subroutine GETGSE and  
C		its subprograms compute values of the emissivity and absorption 
C         coefficients, j and K, resp., from the gyrosynchrotron 
C         radiation formulas.  This program uses four subprograms,
C		based upon subprograms found in "Numerical recipies in
C		FORTRAN 77".  The first is ODEINT, which uses an adaptive step
C		size Runge-Kutta method to integrate one or more ordinary 
C		differential equations.  The second and third are DMLIN and DCADRE.  
C		These two evaluate the integrals in the expressions for j
C         and K.  The fourth routine is MMBSJN, which calculates the
C		values of Bessel functions at a given point.  DMLIN uses a
C	    Gaussian Quadrature method, and will use up to 256 Gauss
C	    points to evaluate the integral.  DCADRE employs the method
C         of Romberg extrapolation, and subdivides the interval of
C         integration until the error tolerance is satisfied. MMBSJN
C	    determines the Bessel functions via Steed's method. This 
C		program reports all errors, from any section of the code, to
C	    the user.  However, fatal errors cause the course of the
C	    calculations to abort immediately after the error has been 
C		reported to the user.
C                                                                        
C    Files used -                                                        
C		DCADRE.FOR			Romberg integration routine
C		DMLIN.FOR			Gaussian Quadrature integration routine
C		FDBPL.FOR			Electron distribution function and its derivatives
C		GETGSE.FOR			Core Routines - Compute j & K
C		GETGSE.INC			Include file - common blocks, parameter definitions
C         GYRORT.FOR			Main Program - this file
C		MMBSJN.FOR			Computes values of Bessel functions
C		ODEINT.FOR			Runge-Kutta ODE integration routine
C
C    Terminal I/O - 		                                             
C         IUNITA     Defines device unit number for reading information  
C                    interactively from the user terminal keyboard.      
C                    Currently assigned to unit number 5.                 
C         IUNITB     Defines device unit number for writing information  
C                    to user's terminal screen.  Currently assigned to   
C                    unit number 6.                                      
C                                                                        
C    Subroutines - 
C		BGAUSS	   Magnetic field strength as a function of position
C		DERIVS	   Equation of radiative transfer.  Calls GETGSE.
C         DISTRN     Electron distribution function and its first derivatives                          
C         GETGSE     Governs computation of gyrosynchrotron coefficients
C		ODEINT     Integrates the equation of radiative transfer
C
C    References - 
C         gyrortdoc.pdf and gyrodoc.pdf

  
      IMPLICIT NONE
  
      INCLUDE     'GETGSE.INC'
  
      INTEGER*4   IERGQ, IERROM, INU, TIME0, ITIME, ITICK, IERALL

	INTEGER*4   ITHETA, NVAR, KMAX, KOUNT, NOK, NBAD, IP

	PARAMETER	(NVAR = 2)
  
      REAL*8      VALUEJ, VALUEK, VALJEX, VALKEX, VALJOR, VALKOR, RNUX

	REAL*8		RADIEX, RADIOR, FLUXEX, FLUXOR, TBRIEX, TBRIOR, RADPOL

	REAL*8		GAMLOW, GAMHIGH, GAMBRK

	REAL*8		BFIELD, BMIN, BMAX, YSTART(NVAR), RINDX0, DEPTH

	REAL*8		DXSAV, TAUEX, TAUOR

	REAL*8		NTH, AREA

	EXTERNAL	DERIVS, RKQS

	COMMON		/DISTLIMS/GAMLOW, GAMHIGH, GAMBRK

	COMMON		IERGQ, IERROM, IERALL

	COMMON		/FIELD/ BMIN, BMAX, DEPTH

      COMMON		/PATH/ KMAX,KOUNT,DXSAV

C				(Must match PATH common block in ODEINT)

C	*****************************************************************
	NAMELIST/input/BMIN,BMAX,NTH,RNUP,DENSTY,P,Q,THETAD,DEPTH,
     1			   GAMLOW,GAMBRK,GAMHIGH,RERR,METHOD
C	*****************************************************************
  
C	OPEN(unit=10,file="harmview.txt", status="REPLACE")
	OPEN(unit=11,file="output.txt", status="REPLACE")
      OPEN(unit=12,file="error.txt", status="REPLACE")

      CALL SYSTEM_CLOCK(TIME0)

C			  ***Input parameters***

	GAMLOW	= 1.02D+00
	GAMBRK	= 1.20D+00
	GAMHIGH	= 11.0D+00

	BMIN	= 400.D+00
	BMAX	= 600.D+00
	THETAD  = 45.00D+00
	DENSTY	= 2.0D+07
	NTH		= 2.0D+09
	P		= 7.0D+00
	Q	    = 5.0D+00
	DEPTH	= 2.0D+09
C			  Source area for flux calculation
	AREA	= 2.5D+00*DEPTH**2

C			  ***Specify method of integration and relative error***

	METHOD  = 1
	RERR    = 0.001D+00

C				 Initialize error counter to zero.  
	IERALL = 0

C				 Maximum number of intermediate values that may be
C				 stored by ODEINT.  Set KMAX to 0 for none.  Otherwise
C				 set KMAX = KMAXX (200).  Intermediate values of X, 
C				 TAU, and I/n**2 are printed to the screen by ODEINT.  
	KMAX	= 200

C				 Minimum interval for which intermediate results are
C				 stored by ODEINT.  KMAX and DXSAV do not affect the
C				 calculated results.  
	DXSAV = DEPTH/10.D+00

C				 Calculate the electron plasma frequency

	RNUP = 8.98D+03*DSQRT(NTH)
	

C				 Loop over observation angle to magnetic field (THETAD)

      DO ITHETA = 18, 18
	   
	   THETAD = DFLOAT(ITHETA) * 2.5D+00

C				 Loop over radio frequencies (RNU)

	DO INU = 0, 430

	   IF( (INU .GE. 0) .AND. (INU .LE. 60) ) THEN
	      RNU = DFLOAT(INU) * 0.1D+08 + 4.0D+08
         ELSE IF( (INU .GE. 61) .AND. (INU .LE. 360) ) THEN
	      RNU = DFLOAT(INU-60) * 0.01D+09 + 1.0D+09
         ELSE IF( (INU .GE. 361) .AND. (INU .LE. 420) ) THEN
	      RNU = DFLOAT(INU-360) * 0.1D+09 + 4.0D+09
         ELSE IF( (INU .GE. 421) .AND. (INU .LE. 430) ) THEN
	      RNU = DFLOAT(INU-420) * 0.1D+10 + 1.0D+10
	   ENDIF

	RADIEX  = 0.00D+00
	RADIOR  = 0.00D+00

C				 *** EXTRAORDINARY MODE CALCULATIONS ***

	   MODE = -1

	 	 YSTART(1) = 0.0D+00
		 YSTART(2) = 0.0D+00

C				 Calculate the index of refraction at x = 0.

		 CALL BGAUSS (0.0D+00, BFIELD)
		 RNUB = 2.80D+06*BFIELD
		 CALL GETIND ()
		 RINDX0 = RINDX

		 CALL ODEINT(YSTART,NVAR,0.0D+00,DEPTH,RERR,1.0D-03*DEPTH,
     *		0.0D00,NOK,NBAD,DERIVS,RKQS)

		 TAUEX	= YSTART(1)
		 RADIEX = YSTART(2) * RINDX0**2

C				 *** ORDINARY MODE CALCULATIONS ***

	   MODE = 1

	 	 YSTART(1) = 0.0D+00
		 YSTART(2) = 0.0D+00

C				 Calculate the index of refraction at x = 0.

		 CALL BGAUSS (0.0D+00, BFIELD)
		 RNUB = 2.80D+06*BFIELD
		 CALL GETIND ()
		 RINDX0 = RINDX

		 CALL ODEINT(YSTART,NVAR,0.0D+00,DEPTH,RERR,1.0D-03*DEPTH,
     *		0.0D00,NOK,NBAD,DERIVS,RKQS)

		 TAUOR	= YSTART(1)
		 RADIOR = YSTART(2) * RINDX0**2

C				 Calculate radio flux in SFU and brightness temperature

	   FLUXEX = RADIEX * AREA / 2.238D+07
	   TBRIEX = 6.50931D+36 * RADIEX / RNU**2
	   FLUXOR = RADIOR * AREA / 2.238D+07
	   TBRIOR = 6.50931D+36 * RADIOR / RNU**2

C				 Calculate fractional polarization

	   RADPOL = (RADIOR - RADIEX) / (RADIOR + RADIEX)

C				 Write results to screen and files

	   WRITE(IUNITB,'(2ES9.2,4(3X,I10))') THETAD, RNU, NSMIN, NSMAX,  
     1                                      NSCON, IERALL
	   WRITE(11,'(11ES14.4)')  THETAD, RNU, TAUEX, TAUOR, 
     1   RADIEX, RADIOR, FLUXEX, FLUXOR, TBRIEX, TBRIOR, RADPOL
	   WRITE(12,'(ES14.4,12I5)') RNU, IERATH, IERIND, IERNS, IERGAM,
     1   IEREMI, IERGQ, IERPHI, IERXS, IERCAT, IERBES, IERROM, IERNU

	END DO
	END DO

      CALL SYSTEM_CLOCK(ITIME, ITICK)
      IF (ITICK /= 0)                                                  
     1WRITE (IUNITB,'(" Processing complete in ", F9.3, " seconds")')       
     2      REAL(ITIME-TIME0)/REAL(ITICK)

C	Write the NAMELIST parameters to the output file.
	WRITE(11,NML=input)
  
      STOP '0000'
      END

	SUBROUTINE DERIVS (X,Y,DYDX)

C		The formal solution to the equation of radiative transfer
C		is written as two coupled equations: one for the optical 
C		depth (tau) and the other for the radiation intensity 
C		divided by the index of refraction squared (I/n**2).

	IMPLICIT NONE

	INCLUDE 'GETGSE.INC'

	INTEGER*4	IERGQ, IERROM, IERALL

	REAL*8		X, Y(*), DYDX(*)
	REAL*8		GAMLOW, GAMHIGH, GAMBRK
	REAL*8		BFIELD, RNUX
	REAL*8		VALJ0, VALK0, VALJ1, VALK1, VALJ2, VALK2
	REAL*8		VALUEJ, VALUEK, VALJ, VALK

	COMMON		/DISTLIMS/GAMLOW, GAMHIGH, GAMBRK
	COMMON		IERGQ, IERROM, IERALL

	CALL BGAUSS (X, BFIELD)
	RNUB = 2.80D+06*BFIELD
      RNUX = DSQRT( (RNUP**2) + ((RNUB/2.0D+00)**2) )
      RNUX = RNUX + (RNUB/2.0D+00)

	   IF( (RNU .LE. RNUP) .OR. (RNU .LE. RNUX .AND. MODE .EQ. -1) ) 
     *    THEN

		 DYDX(1) = 0.0D+00
		 DYDX(2) = 0.0D+00

	   ELSE
C                  Call to subroutine GETGSE, to get the values for the
C                  gyrosynchrotron emission and absorption coefficients.
C			     This will be performed in two steps: 
C				 once for GAMMA between GAMLOW and GAMBRK,
C                  and again for GAMMA between GAMBRK and GAMHIGH.  The
C                  results for these values of J and K will be added 
C                  together to obtain the final values of J and K.

           GAMMIN = GAMLOW
	     GAMMAX = GAMBRK
           CALL GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )
	     VALJ1 = VALUEJ
	     VALK1 = VALUEK

C                  Check for any errors or warnings.
	     IF ( (IERATH .NE. 0) .OR. (IERIND .NE. 0) .OR. 
     1	       (IERNS .NE. 0) .OR. (IERGAM .NE. 0) .OR. 
     2		  (IEREMI .NE. 0) .OR. (IERGQ .NE. 0)  .OR.
     3          (IERPHI .NE. 0) .OR. (IERXS .NE. 0)  .OR. 
     4          (IERCAT .NE. 0) .OR. (IERBES .EQ. 140) .OR.
     5          (IERBES .EQ. 141) .OR. (IERBES .EQ.142) .OR.  
     6		  (IERROM .NE. 0) .OR. (IERNU .NE. 0) ) THEN
	        IERALL = IERALL + 1
	     ENDIF

	     GAMMIN = GAMBRK
           GAMMAX = GAMHIGH
	     CALL GETGSE ( VALUEJ, VALUEK, IERGQ, IERROM )
	     VALJ2 = VALUEJ
	     VALK2 = VALUEK

C                  Check for any errors or warnings.
	     IF ( (IERATH .NE. 0) .OR. (IERIND .NE. 0) .OR. 
     1	       (IERNS .NE. 0) .OR. (IERGAM .NE. 0) .OR. 
     2		  (IEREMI .NE. 0) .OR. (IERGQ .NE. 0)  .OR.
     3          (IERPHI .NE. 0) .OR. (IERXS .NE. 0)  .OR. 
     4          (IERCAT .NE. 0) .OR. (IERBES .EQ. 140) .OR.
     5          (IERBES .EQ. 141) .OR. (IERBES .EQ.142) .OR.  
     6		  (IERROM .NE. 0) .OR. (IERNU .NE. 0) ) THEN
	        IERALL = IERALL + 1
	     ENDIF

	     VALJ = VALJ1 + VALJ2
	     VALK = VALK1 + VALK2

C			 The radiative transfer equations: 
C			   d(tau)/dx    = kappa
C			   d(I/n**2)/dx = (j/n**2) * exp(-tau)

		 DYDX(1) =  VALK

		IF ( Y(1) .LT. -70.0D+00 ) THEN

C			 Avoid stopping the program with an overflow

		 DYDX(2) = 1.0D+30

		ELSE

		 DYDX(2) = (VALJ / RINDX**2) * DEXP(-Y(1))

		END IF

	   END IF

	RETURN
	END

	SUBROUTINE BGAUSS (X, BFIELD)

C			Returns the value of the magnetic field strength 
C			in Gauss at position x

	IMPLICIT NONE

	REAL*8		X, BFIELD
	REAL*8		BMIN, BMAX, DEPTH

	COMMON		/FIELD/ BMIN, BMAX, DEPTH

	BFIELD = BMIN + (BMAX - BMIN) * (X / DEPTH)

	RETURN
	END